import os

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf import api_sign
from edge.webserver.sysconf import api_sign_iit


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/sign   /<uiref>
    PUT   /api/v2/sys/conf/sign   /<uiref>
    PUT   /api/v2/sys/conf/sign   /<uiref>/install
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._sign__get_uiref,
            ],
            'PUT': [
                cls._sign__put_uiref,
                cls._sign__put_uiref_install
            ],
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _sign__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf_pub = api_sign_iit.load_unit()
            return cgi.CgiReply(body=ext_uiconf_pub)

        return cls(handler)

    @classmethod
    def _sign__put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf_pub = api_sign_iit.save_unit(request.body_jnode)
            return cgi.CgiReply(body=ext_uiconf_pub)

        return cls(handler)

    @classmethod
    def _sign__put_uiref_install(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        raise cgi.CgiError400 # TODO delete function ?

        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'install':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        crt_path = os.getenv('CRT_PATH')
        if crt_path is None:
            eventlog.log_d('fail: http400 (env.CRT_PATH not found)')
            raise cgi.CgiError400

        key_path = os.getenv('KEY_PATH')
        if key_path is None:
            eventlog.log_d('fail: http400 (env.KEY_PATH not found)')
            raise cgi.CgiError400

        key_pass = os.getenv('KEY_PASS')

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf_pub = api_sign.install_crt_key(crt_path, key_path, key_pass)
            return cgi.CgiReply(body=ext_uiconf_pub)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-sign', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
