# coding=utf-8
from edge.iit import EUSignCP
from edge.com import fs
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.uimod.sign import SignUimod
from edge import eventlog


def load_unit():
    """
    :rtype: dict
    """
    SignUimod.install()
    uiconf = SignUimod.load_conf()

    ext_uiconf_pub = uiconf['rw']
    ext_uiconf_pub['conf']['key'].pop('password', None)
    ext_uiconf_pub['conf']['proxy'].pop('auth-pass', None)
    reply = {
        'type': ph.UI_TYPE_SIGN,
        'conf': ext_uiconf_pub['conf'],
        'limits': _get_limits()
    }
    return reply


def _get_limits():
    EUSignCP.EULoad()
    ctx = EUSignCP.EUGetInterface()  # type:EUSignCP.EU_INTERFACE
    ctx.Initialize()

    description = list()
    device = list()
    media_type_index = 0
    limits_jnode = {
        "key": []
    }
    while ctx.EnumKeyMediaTypes(media_type_index, description):
        if description[0] != u'е.ключ ІІТ Алмаз-1К':
            media_type_index += 1
            continue

        # print media_type_index, description[0]

        device_index = 0
        while ctx.EnumKeyMediaDevices(media_type_index, device_index, device):
            # print media_type_index, device_index, device[0]
            limits_jnode['key'].append({
                "model": description[0],
                "serial": device[0]
            })
            device_index += 1

        media_type_index += 1
        # break

    return limits_jnode


def _edit_iit_conf(new_uiconf_conf):
    """
    :type new_uiconf_conf: dict
    :return:
    """
    EUSignCP.EULoad()
    ctx = EUSignCP.EUGetInterface()  # type:EUSignCP.EU_INTERFACE
    # ctx.SetUIMode(1)
    ctx.Initialize()

    mode_config = {
        "bOfflineMode": False
    }
    ctx.SetModeSettings(mode_config)

    jnode_fs = new_uiconf_conf['crl']

    file_store_settings = dict()
    try:
        ctx.GetFileStoreSettings(file_store_settings)
    except RuntimeError:
        eventlog.log_d('fail: GetFileStoreSettings(). Creating settings')

    file_store_settings['szPath'] = '/data/certificates'
    file_store_settings['bCheckCRLs'] = jnode_fs['enable']
    file_store_settings['bAutoRefresh'] = False
    file_store_settings['bOwnCRLsOnly'] = jnode_fs['only-own-crls']
    file_store_settings['bFullAndDeltaCRLs'] = jnode_fs['full-and-delta-crls']
    file_store_settings['bAutoDownloadCRLs'] = True
    file_store_settings['bSaveLoadedCerts'] = True
    file_store_settings['dwExpireTime'] = 3600

    ctx.SetFileStoreSettings(file_store_settings)

    # print 'file_store_settings', file_store_settings

    jnode_proxy = new_uiconf_conf['proxy']
    jnode_proxy_auth_type = jnode_proxy['auth-type']
    proxy_setting = dict()
    try:
        ctx.GetProxySettings(proxy_setting)
    except RuntimeError:
        eventlog.log_i('fail: GetProxySettings(). Creating settings')

    proxy_setting['bUseProxy'] = jnode_proxy.get('enable', False)
    proxy_setting['szAddress'] = u'{}'.format(jnode_proxy['host'])
    proxy_setting['szPort'] = u'{}'.format(jnode_proxy['port'])
    if jnode_proxy_auth_type == 'anonymous':
        proxy_setting['bAnonymous'] = True
        proxy_setting['szUser'] = u''
        proxy_setting['szPassword'] = u''
        proxy_setting['bSavePassword'] = False

    elif jnode_proxy_auth_type == 'credentials':
        proxy_setting['bAnonymous'] = False
        proxy_setting['szUser'] = u'{}'.format(jnode_proxy['auth-user'])
        proxy_setting['szPassword'] = u'{}'.format(jnode_proxy['auth-pass'])
        proxy_setting['bSavePassword'] = True

    ctx.SetProxySettings(proxy_setting)

    # print 'proxy_setting', proxy_setting

    jnode_cmp = new_uiconf_conf['cmp']
    cmp_settings = dict()
    try:
        ctx.GetCMPSettings(cmp_settings)
    except RuntimeError:
        eventlog.log_i('fail: GetCMPSettings(). Creating settings')

    cmp_settings['szPort'] = u'{}'.format(jnode_cmp['port'])
    cmp_settings['szAddress'] = u'{}'.format(jnode_cmp['host'])
    # cmp_settings['szCommonName'] = u'name'
    cmp_settings['bUseCMP'] = True
    ctx.SetCMPSettings(cmp_settings)

    # print 'cmp_settings', cmp_settings

    jnode_oscp = new_uiconf_conf['ocsp']
    oscp_settings = dict()
    try:
        ctx.GetOCSPSettings(oscp_settings)
    except RuntimeError:
        eventlog.log_i('fail: GetOCSPSettings(). Creating settings')
    oscp_settings['bUseOCSP'] = jnode_oscp.get('enable', False)
    oscp_settings['szAddress'] = u'{}'.format(jnode_oscp['host'])
    oscp_settings['szPort'] = u'{}'.format(jnode_oscp['port'])
    oscp_settings['bBeforeStore'] = False
    ctx.SetOCSPSettings(oscp_settings)
    # print 'oscp_settig', oscp_settig

    jnode_tsp = new_uiconf_conf['tsp']
    tsp_settings = dict()
    try:
        ctx.GetTSPSettings(tsp_settings)
    except RuntimeError:
        eventlog.log_i('fail: GetTSPSettings(). Creating settings')
    tsp_settings['szAddress'] = u'{}'.format(jnode_tsp['host'])
    tsp_settings['szPort'] = u'{}'.format(jnode_tsp['port'])
    tsp_settings['bGetStamps'] = True
    ctx.SetTSPSettings(tsp_settings)
    # print 'tsp_settings', tsp_settings

    # IIT library doesn't work if LDAP settings are not set
    ldap_settings = dict()
    try:
        ctx.GetLDAPSettings(ldap_settings)
    except RuntimeError:
        eventlog.log_i('fail: GetLDAPSettings(). Creating stub settings')
    ldap_settings['bUseLDAP'] = False
    ldap_settings['szAddress'] = u''
    ldap_settings['szPort'] = u''
    ldap_settings['bAnonymous'] = True
    ldap_settings['szUser'] = u''
    ldap_settings['szPassword'] = u''
    ctx.SetLDAPSettings(ldap_settings)

    description = list()
    device = list()
    media_type_index = 1
    limits_jnode = {
        "key": []
    }
    while ctx.EnumKeyMediaTypes(media_type_index, description):
        if description[0] != u'е.ключ ІІТ Алмаз-1К':
            media_type_index += 1
            continue
        # print media_type_index, description

        device_index = 0
        while ctx.EnumKeyMediaDevices(media_type_index, device_index, device):
            # print media_type_index, device_index, device
            limits_jnode['key'].append({
                "model": description[0],
                "serial": device[0]
            })
            device_index += 1

        media_type_index += 1

    return limits_jnode
    # ctx.GetPrivateKeyMedia()

    # cmp_settings2 = dict()
    # ctx.GetCMPSettings(cmp_settings2)
    # print cmp_settings2


def save_unit(new_uiconf):
    """
    :type new_uiconf: dict
    :return:
    """
    new_uiconf_conf = new_uiconf.get('conf')

    def validate_body():
        # noinspection PyTypeChecker
        fs.validate_json({'conf': new_uiconf_conf}, SignUimod.schema_path())

    validate_body()

    # edit ui jnode
    uiconf = SignUimod.load_conf()
    uiconf_pub = uiconf.get('rw')

    new_pkey_pass = new_uiconf_conf['key'].get('password')
    new_proxy_pass = new_uiconf_conf['proxy'].get('auth-pass')

    if new_pkey_pass is None:
        new_uiconf_conf['key']['password'] = uiconf_pub['conf']['key']['password']

    if new_proxy_pass is None:
        new_uiconf_conf['proxy']['auth-pass'] = uiconf_pub['conf']['proxy'].get('auth-pass')

    # edit iit config
    limits_jnode = _edit_iit_conf(new_uiconf_conf)

    uiconf_pub['conf'] = new_uiconf_conf
    SignUimod.save_conf(uiconf)

    # regenerate speedcontrol configs
    from edge.webserver.sysconf.api_traffic_rules import ApiTrafficRules

    root_info = ph.RootInfoUiconf.get_info()
    uimods = ph.InfoUiconf.map_units_r(
        root_info, lambda x: ph.UiMod.create(x['ref']),
        unit_types=[ph.UI_TYPE_TRAFFIC_RULES])

    for uimod in uimods:
        ApiTrafficRules.save_unit(uimod.ref, dict())

    new_uiconf_conf['key'].pop('password', None)
    new_uiconf_conf['proxy'].pop('auth-pass', None)
    reply = {
        'type': ph.UI_TYPE_SIGN,
        'conf': new_uiconf_conf,
        'limits': limits_jnode
    }
    return reply


if __name__ == '__main__':
    jnodes = {
        "conf": {
            "active": True,
            "proxy": {
                "enable": False,
                "host": u"Привет",
                "port": 1234,
                "auth-type": "credentials",
                "auth-user": "1234",
                "auth-pass": "1234"
            },
            "cmp": {
                "host": "ca.iit.com.uah",
                "port": 80
            },
            "tsp": {
                "host": "ca.iit.com.uah",
                "port": 80
            },
            "crl": {
                "enable": True,
                "full-and-delta-crls": True,
                "only-own-crls": False
            },
            "ocsp": {
                "enable": True,
                "host": "example.com",
                "port": 1234
            }}
    }

    _edit_iit_conf(jnodes['conf'])
    # reply = _get_limits()
    # print '------------------------------------'
    # print 'reply: ', reply
